@@ -1345,3 +1345,52 @@ test_that("GraphLearner other properties", {
   expect_true(all(c("loglik", "oob_error") %in% g_nested$properties))
 
 })
+
+test_that("GraphLearner - predict_newdata_fast", {
+  # Classification
+  task = tsk("iris")
+  newdata = task$data(cols = task$feature_names, rows = 1:10)
+  
+  ## predict_type = "response"
+  learner = lrn("classif.featureless")
+  set.seed(20251117)
+  lrn_pred = learner$train(task)$predict_newdata_fast(newdata)
+  set.seed(20251117)
+  glrn_pred = as_learner(po("learner", learner))$train(task)$predict_newdata_fast(newdata)
+  
+  expect_equal(as.character(glrn_pred$response), lrn_pred$response)
+  expect_equal(glrn_pred$prob, lrn_pred$prob)
+  
+  ## predict_type = "prob"
+  learner = lrn("classif.featureless", predict_type = "prob")
+  set.seed(20251117)
+  lrn_pred = learner$train(task)$predict_newdata_fast(newdata)
+  set.seed(20251117)
+  glrn_pred = as_learner(po("learner", learner))$train(task)$predict_newdata_fast(newdata)
+  
+  # Some Learners only return probs if predict_type is "prob" in .predict() and labels are inferred later, 
+  # so there is an acceptable difference between the two methods. 
+  expect_true(identical(glrn_pred$response, lrn_pred$response) || is.null(lrn_pred$response))
+  expect_equal(glrn_pred$prob, lrn_pred$prob)
+
+  # Regression
+  task = tsk("mtcars")
+  newdata = task$data(cols = task$feature_names, rows = 1:10)
+  ## predict_type = "response"
+  learner = lrn("regr.featureless")
+  lrn_pred = learner$train(task)$predict_newdata_fast(newdata)
+  glrn_pred = as_learner(po("learner", learner))$train(task)$predict_newdata_fast(newdata)
+  expect_equal(glrn_pred, lrn_pred)
+
+  ## predict_type = "se"
+  learner = lrn("regr.featureless", predict_type = "se")
+  lrn_pred = learner$train(task)$predict_newdata_fast(newdata)
+  glrn_pred = as_learner(po("learner", learner))$train(task)$predict_newdata_fast(newdata)
+  expect_equal(glrn_pred, lrn_pred)
+
+  ## predict_type = "quantiles"
+  learner = lrn("regr.featureless", predict_type = "quantiles", quantiles = 0.25)
+  lrn_pred = learner$train(task)$predict_newdata_fast(newdata)
+  glrn_pred = as_learner(po("learner", learner))$train(task)$predict_newdata_fast(newdata)
+  expect_equal(glrn_pred$quantiles[, 1L], lrn_pred$quantiles[, 1L])
+})
