@@ -1,5 +1,7 @@
 context("PipeOpEnsemble")
 
+skip_on_cran()
+
 test_that("PipeOpEnsemble - basic properties", {
   op = PipeOpEnsemble$new(4, id = "ensemble", param_vals = list())
   expect_pipeop(op)
@@ -69,6 +71,7 @@ test_that("PipeOpWeightedRegrAvg - train and predict", {
 
 test_that("PipeOpWeightedClassifAvg - response - train and predict", {
   nulls = rep(list(NULL), 4)
+  set.seed(1L)
   prds = replicate(4,
     make_prediction_obj_classif(n = 100, noise = TRUE,
       predict_types = "response", nclasses = 3),
@@ -103,6 +106,7 @@ test_that("PipeOpWeightedClassifAvg - response - train and predict", {
 })
 
 test_that("PipeOpWeightedClassifAvg - prob - train and predict", {
+  set.seed(2L)
   nulls = rep(list(NULL), 4)
   prds = replicate(4,
     make_prediction_obj_classif(n = 100, noise = TRUE,
